<?php

namespace Core\Exceptions;

use Core\Http\Request;
use Throwable;
use Whoops\Handler\Handler;
use Whoops\Handler\JsonResponseHandler;
use Whoops\Handler\PlainTextHandler;
use Whoops\Handler\PrettyPageHandler;
use Whoops\Run as Whoops;

class ExceptionHandler
{
    /**
     * Exceptions handler instance
     *
     * @var Whoops
     */
    private static Whoops $instance;

    /**
     * Get exceptions handler instance
     *
     * @param \Core\Http\Request|null $request
     * @return \Whoops\Run
     */
    public static function instance(Request|null $request = null): Whoops
    {
        if (!isset(self::$instance)) {
            $instance = new Whoops();
            $instance->pushHandler(self::handler($request));

            self::$instance = $instance;
        }

        return self::$instance;
    }

    /**
     * Get correct handler
     *
     * @return \Whoops\Handler\Handler
     */
    private static function handler(): Handler
    {
        if (env('debug')) {
            if (request()?->is('post')) {
                return new JsonResponseHandler();
            }

            return new PrettyPageHandler();
        }

        return new PlainTextHandler();
    }

    /**
     * Catch all exceptions
     *
     * @return void
     */
    public static function catch(): void
    {
        self::instance()->register();
    }

    /**
     * Catch single exception
     *
     * @param \Throwable $exception
     * @return never
     */
    public static function catchOne(Throwable $exception): never
    {
        self::instance()->handleException($exception);

        exit(0);
    }

    /**
     * Make new exception
     *
     * @param null        $abstract
     * @param string|null $message
     * @return never
     */
    public static function make(mixed $abstract = null, string|null $message = null): never
    {
        if(is_string($abstract)) {
            $abstract = app()->make($abstract, $message);
        }

        if(is_subclass_of($abstract, 'Exception')) {
            self::catchOne($abstract);
        }

        exit(0);
    }

}